@@ -47,60 +47,14 @@
 #define EPHY_EMBED_SINGLE_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_EMBED_SINGLE, EphyEmbedSinglePrivate))
 
 struct _EphyEmbedSinglePrivate {
-  guint online : 1;
-
   GHashTable *form_auth_data;
   SoupCache *cache;
 };
 
-enum {
-  PROP_0,
-  PROP_NETWORK_STATUS
-};
-
 static void ephy_embed_single_init (EphyEmbedSingle *single);
 static void ephy_embed_single_class_init (EphyEmbedSingleClass *klass);
 static void ephy_permission_manager_iface_init (EphyPermissionManagerIface *iface);
 
-static void
-ephy_embed_single_get_property (GObject *object,
-                                guint prop_id,
-                                GValue *value,
-                                GParamSpec *pspec)
-{
-  EphyEmbedSingle *single = EPHY_EMBED_SINGLE (object);
-
-  switch (prop_id) {
-  case PROP_NETWORK_STATUS:
-    g_value_set_boolean (value, ephy_embed_single_get_network_status (single));
-    break;
-  default:
-    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-    break;
-  }
-}
-
-static void
-ephy_embed_single_set_property (GObject *object,
-                                guint prop_id,
-                                const GValue *value,
-                                GParamSpec *pspec)
-{
-  EphyEmbedSingle *single = EPHY_EMBED_SINGLE (object);
-
-  switch (prop_id) {
-  case PROP_NETWORK_STATUS:
-    ephy_embed_single_set_network_status (single, g_value_get_boolean (value));
-    break;
-  default:
-    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-    break;
-  }
-}
-
-/* Some compilers (like gcc 2.95) don't support preprocessor directives inside macros,
-   so we have to duplicate the whole thing */
-
 G_DEFINE_TYPE_WITH_CODE (EphyEmbedSingle, ephy_embed_single, G_TYPE_OBJECT,
                          G_IMPLEMENT_INTERFACE (EPHY_TYPE_PERMISSION_MANAGER,
                                                 ephy_permission_manager_iface_init))
@@ -242,8 +196,6 @@ ephy_embed_single_finalize (GObject *object)
 {
   EphyEmbedSinglePrivate *priv = EPHY_EMBED_SINGLE (object)->priv;
 
-  ephy_embed_prefs_shutdown ();
-
   if (priv->form_auth_data) {
     g_hash_table_foreach (priv->form_auth_data,
                           (GHFunc)remove_form_auth_data,
@@ -260,7 +212,6 @@ ephy_embed_single_init (EphyEmbedSingle *single)
   EphyEmbedSinglePrivate *priv;
 
   single->priv = priv = EPHY_EMBED_SINGLE_GET_PRIVATE (single);
-  priv->online = TRUE;
 
   priv->form_auth_data = g_hash_table_new_full (g_str_hash,
                                                 g_str_equal,
@@ -276,8 +227,6 @@ ephy_embed_single_class_init (EphyEmbedSingleClass *klass)
 
   object_class->finalize = ephy_embed_single_finalize;
   object_class->dispose = ephy_embed_single_dispose;
-  object_class->get_property = ephy_embed_single_get_property;
-  object_class->set_property = ephy_embed_single_set_property;
 
   /**
    * EphyEmbedSingle::new-window:
@@ -324,20 +273,6 @@ ephy_embed_single_class_init (EphyEmbedSingleClass *klass)
                 G_TYPE_STRING,
                 G_TYPE_STRING);
 
-  /**
-   * EphyEmbedSingle::network-status:
-   * 
-   * Whether the network is on-line.
-   */
-  g_object_class_install_property
-    (object_class,
-     PROP_NETWORK_STATUS,
-     g_param_spec_boolean ("network-status",
-                           "network-status",
-                           "network-status",
-                           FALSE,
-                           G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
-
   g_type_class_add_private (object_class, sizeof (EphyEmbedSinglePrivate));
 }
 
@@ -413,6 +348,7 @@ ephy_embed_single_initialize (EphyEmbedSingle *single)
   char *filename;
   char *cookie_policy;
   char *cache_dir;
+  char *favicon_db_path;
   EphyEmbedSinglePrivate *priv = single->priv;
   SoupSessionFeature *requester;
 
@@ -420,8 +356,6 @@ ephy_embed_single_initialize (EphyEmbedSingle *single)
   if (g_file_test (NSPLUGINWRAPPER_SETUP, G_FILE_TEST_EXISTS) != FALSE)
     g_spawn_command_line_sync (NSPLUGINWRAPPER_SETUP, NULL, NULL, NULL, NULL);
 
-  ephy_embed_prefs_init ();
-
   session = webkit_get_default_session ();
 
   /* Check SSL certificates */
@@ -475,6 +409,11 @@ ephy_embed_single_initialize (EphyEmbedSingle *single)
     soup_session_add_feature_by_type (session, SOUP_TYPE_PASSWORD_MANAGER_GNOME);
 #endif
 
+  /* Initialize the favicon cache. */
+  favicon_db_path = g_build_filename (g_get_user_data_dir (), g_get_prgname (), NULL);
+  webkit_favicon_database_set_path (webkit_get_favicon_database (), favicon_db_path);
+  g_free (favicon_db_path);
+
   return TRUE;
 }
 
@@ -509,37 +448,6 @@ ephy_embed_single_clear_auth_cache (EphyEmbedSingle *single)
 {
 }
 
-/**
- * ephy_embed_single_get_nework_status:
- * @single: the #EphyEmbedSingle
- * @offline: %TRUE if the network is on-line
- * 
- * Sets the state of the network connection.
- **/
-void
-ephy_embed_single_set_network_status (EphyEmbedSingle *single,
-                                      gboolean status)
-{
-  if (status != single->priv->online)
-    single->priv->online = status;
-
-  g_object_notify (G_OBJECT (single), "network-status");
-}
-
-/**
- * ephy_embed_single_get_network_status:
- * @single: the #EphyEmbedSingle
- * 
- * Gets the state of the network connection.
- * 
- * Returns: %TRUE iff the network is on-line.
- **/
-gboolean
-ephy_embed_single_get_network_status (EphyEmbedSingle *single)
-{
-  return single->priv->online;
-}
-
 /**
  * ephy_embed_single_open_window:
  * @single: the #EphyEmbedSingle
@@ -580,7 +488,7 @@ ephy_embed_single_open_window (EphyEmbedSingle *single,
  * The #EphyEmbedSingleFormAuthData structs and the #GSList are owned
  * by @single and should not be freed by the user.
  * 
- * Returns: (transfer none): #GSList with the possible auto-fills for the forms
+ * Returns: (transfer none) (element-type EphyEmbedSingleFormAuthData): #GSList with the possible auto-fills for the forms
  * in @uri, or %NULL
  **/
 GSList *
