@@ -164,14 +164,17 @@ ephy_file_desktop_dir (void)
 
 /**
  * ephy_file_tmp_filename:
- * @base: the base name of the temp file to create
+ * @base: the base name of the temp file to create, containing "XXXXXX"
  * @extension: an optional extension for @base or %NULL
  *
- * Creates a temp file with mkstemp() using @base as the name with an optional
- * @extension.
+ * Gets a usable temp filename with g_mkstemp() using @base as the name
+ * with an optional @extension. @base should contain "XXXXXX" in it.
  *
- * Returns: a newly-allocated string containing the name of the created temp
- * file or %NULL.
+ * Notice that this does not create the file. It only gets a valid
+ * filename.
+ *
+ * Returns: a newly-allocated string containing the name of the temp
+ * file name or %NULL.
  **/
 char *
 ephy_file_tmp_filename (const char *base,
@@ -224,15 +227,9 @@ ephy_file (const char *filename)
 	{
 #ifdef GNOME_ENABLE_DEBUG
 		TOP_SRC_DATADIR "/",
-		TOP_SRC_DATADIR "/glade/",
-		TOP_SRC_DATADIR "/ui/",
-		TOP_SRC_DATADIR "/art/",
 		TOP_SRC_DATADIR "/pages/",
 #endif
 		SHARE_DIR "/",
-		SHARE_DIR "/glade/",
-		SHARE_DIR "/ui/",
-		SHARE_DIR "/art/",
 		SHARE_DIR "/pages/"
 	};
 
@@ -396,6 +393,8 @@ ephy_file_helpers_shutdown (void)
 		g_free (tmp_dir);
 		tmp_dir = NULL;
 	}
+
+	g_unsetenv (EPHY_UUID_ENVVAR);
 }
 
 /**
@@ -510,6 +509,7 @@ ephy_file_switch_temp_file (GFile *file_dest,
 	gboolean dest_exists;
 	gboolean retval = TRUE;
 	GFile *backup_file;
+	GError *error = NULL;
 
 	file_dest_path = g_file_get_path (file_dest);
 	file_temp_path = g_file_get_path (file_temp);
@@ -523,10 +523,17 @@ ephy_file_switch_temp_file (GFile *file_dest,
 	{
 		if (g_file_move (file_dest, backup_file,
 				 G_FILE_COPY_OVERWRITE,
-				 NULL, NULL, NULL, NULL) == FALSE)
+				 NULL, NULL, NULL, &error) == FALSE)
 		{
-			g_warning ("Failed to backup %s to %s",
-				   file_dest_path, backup_path);
+			g_warning ("Failed to backup %s to %s: %s",
+				   file_dest_path, backup_path,
+				   error ? error->message : "No error set");
+
+			if (error)
+			{
+				g_error_free (error);
+				error = NULL;
+			}
 
 			retval = FALSE;
 			goto failed;
@@ -535,17 +542,31 @@ ephy_file_switch_temp_file (GFile *file_dest,
 
 	if (g_file_move (file_temp, file_dest,
 			 G_FILE_COPY_OVERWRITE,
-			 NULL, NULL, NULL, NULL) == FALSE)
+			 NULL, NULL, NULL, &error) == FALSE)
 	{
-		g_warning ("Failed to replace %s with %s",
-			   file_temp_path, file_dest_path);
+		g_warning ("Failed to replace %s with %s: %s",
+			   file_temp_path, file_dest_path,
+			   error ? error->message : "No error set");
+
+		if (error)
+		{
+			g_error_free (error);
+			error = NULL;
+		}
 
 		if (g_file_move (backup_file, file_dest,
 				 G_FILE_COPY_OVERWRITE,
-				 NULL, NULL, NULL, NULL) == FALSE)
+				 NULL, NULL, NULL, &error) == FALSE)
 		{
-			g_warning ("Failed to restore %s from %s",
-				   file_dest_path, file_temp_path);
+			g_warning ("Failed to restore %s from %s: %s",
+				   file_dest_path, file_temp_path,
+				   error ? error->message : "No error set");
+
+			if (error)
+			{
+				g_error_free (error);
+				error = NULL;
+			}
 		}
 
 		retval = FALSE;
@@ -554,10 +575,17 @@ ephy_file_switch_temp_file (GFile *file_dest,
 
 	if (dest_exists)
 	{
-		if (g_file_delete (backup_file,
-				   NULL, NULL) == FALSE)
+		if (g_file_delete (backup_file, NULL, &error) == FALSE)
 		{
-			g_warning ("Failed to delete old file %s", backup_path);
+			g_warning ("Failed to delete old file %s: %s",
+				   backup_path,
+				   error ? error->message : "No error set");
+
+			if (error)
+			{
+				g_error_free (error);
+				error = NULL;
+			}
 		}
 	}
 
@@ -584,24 +612,6 @@ ephy_file_delete_on_exit (GFile *file)
 	/* does nothing now */
 }
 
-/**
- * ephy_file_add_recent_item:
- * @uri: an URI
- * @mime_type: the mime type corresponding to @uri
- *
- * Adds @uri to the default #GtkRecentManager, with @mime_type as its type.
- **/
-void
-ephy_file_add_recent_item (const char *uri,
-			   const char *mime_type)
-{	
-	GtkRecentManager *manager = gtk_recent_manager_get_default ();
-
-	g_return_if_fail (mime_type != NULL && uri != NULL);
-
-	gtk_recent_manager_add_item (manager, uri);
-}
-
 static void
 load_mime_from_xml (void)
 {
@@ -868,18 +878,20 @@ ephy_file_browse_to (GFile *file,
  * @error: location to set any #GError
  *
  * Remove @path and its contents. Like calling rm -rf @path.
+ *
+ * Returns: %TRUE if delete succeeded
  **/
 gboolean
 ephy_file_delete_dir_recursively (GFile *directory, GError **error)
 {
 	GFileEnumerator *children = NULL;
 	GFileInfo *info;
-	gboolean ret = TRUE;
+	gboolean ret = FALSE;
 
 	children = g_file_enumerate_children (directory,
 					      "standard::name,standard::type",
 					      0, NULL, error);
-	if (error)
+	if (children == NULL || error)
 		goto out;
 
 	info = g_file_enumerator_next_file (children, NULL, error);
@@ -899,7 +911,7 @@ ephy_file_delete_dir_recursively (GFile *directory, GError **error)
 		if (type == G_FILE_TYPE_DIRECTORY)
 			ret = ephy_file_delete_dir_recursively (child, error);
 		else if (type == G_FILE_TYPE_REGULAR)
-			ret =	g_file_delete (child, NULL, error);
+			ret = g_file_delete (child, NULL, error);
 
 		g_object_unref (info);
 
