@@ -26,7 +26,6 @@
 #include "ephy-bookmarks-ui.h"
 #include "ephy-debug.h"
 #include "ephy-dnd.h"
-#include "ephy-favicon-cache.h"
 #include "ephy-file-helpers.h"
 #include "ephy-gui.h"
 #include "ephy-hosts-store.h"
@@ -203,7 +202,7 @@ confirmation_dialog_response_cb (GtkWidget *dialog,
 	if (response == GTK_RESPONSE_ACCEPT)
 	{
 		ephy_history_service_clear (editor->priv->history_service,
-					    NULL, NULL);
+					    NULL, NULL, NULL);
 		filter_now (editor, TRUE, TRUE);
 	}
 }
@@ -461,13 +460,13 @@ cmd_delete (GtkAction *action,
 	{
 		GList *selected;
 		selected = ephy_urls_view_get_selection (EPHY_URLS_VIEW (editor->priv->pages_view));
-		ephy_history_service_delete_urls (editor->priv->history_service, selected,
+		ephy_history_service_delete_urls (editor->priv->history_service, selected, NULL,
 						  (EphyHistoryJobCallback)on_browse_history_deleted_cb, editor);
 	} else if (gtk_widget_is_focus (editor->priv->hosts_view)) {
 		EphyHistoryHost *host = get_selected_host (editor);
 		if (host) {
 			ephy_history_service_delete_host (editor->priv->history_service,
-							  host,
+							  host, NULL,
 							  (EphyHistoryJobCallback)on_host_deleted_cb,
 							  editor);
 			ephy_history_host_free (host);
@@ -576,26 +575,23 @@ ephy_history_window_row_activated_cb (GtkTreeView *view,
 	ephy_history_url_free (url);
 }
 
-#if 0
 static void
-ephy_history_window_node_middle_clicked_cb (GtkWidget *view,
-					    EphyNode *node,
-					    EphyHistoryWindow *editor)
+ephy_history_window_row_middle_clicked_cb (EphyHistoryView *view,
+					   GtkTreePath *path,
+					   EphyHistoryWindow *editor)
 {
 	EphyWindow *window;
-	const char *location;
+	EphyHistoryURL *url;
 
 	window = EPHY_WINDOW (get_target_window (editor));
+	url = ephy_urls_store_get_url_from_path (editor->priv->urls_store, path);
+	g_return_if_fail (url != NULL);
 
-	location = ephy_node_get_property_string
-		(node, EPHY_NODE_PAGE_PROP_LOCATION);
-	g_return_if_fail (location != NULL);
-
-	ephy_shell_new_tab (ephy_shell, window, NULL, location,
+	ephy_shell_new_tab (ephy_shell, window, NULL, url->url,
 			    EPHY_NEW_TAB_OPEN_PAGE |
 			    EPHY_NEW_TAB_IN_EXISTING_WINDOW);
+	ephy_history_url_free (url);
 }
-#endif
 
 static void
 ephy_history_window_update_menu (EphyHistoryWindow *editor)
@@ -873,21 +869,22 @@ delete_event_cb (EphyHistoryWindow *editor)
 static void
 provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 {
-	EphyFaviconCache *cache;
-	const char *icon_location;
+	const char *page_location;
 	GdkPixbuf *pixbuf = NULL;
 
-	cache = EPHY_FAVICON_CACHE
-		(ephy_embed_shell_get_favicon_cache (EPHY_EMBED_SHELL (ephy_shell)));
-	icon_location = ephy_node_get_property_string
-		(node, EPHY_NODE_PAGE_PROP_ICON);
+	page_location = ephy_node_get_property_string
+		(node, EPHY_NODE_PAGE_PROP_LOCATION);
 
-	LOG ("Get favicon for %s", icon_location ? icon_location : "None");
+	LOG ("Get favicon for %s", page_location ? page_location : "None");
 
-	if (icon_location)
-	{
-		pixbuf = ephy_favicon_cache_get (cache, icon_location);
-	}
+	if (page_location)
+        {
+		/* No need to use the async version as this function will be
+		called many times by the treeview. */
+		WebKitFaviconDatabase *database = webkit_get_favicon_database ();
+		pixbuf = webkit_favicon_database_get_favicon_pixbuf (database, page_location,
+								     FAVICON_SIZE, FAVICON_SIZE);
+        }
 
 	g_value_init (value, GDK_TYPE_PIXBUF);
 	g_value_take_object (value, pixbuf);
@@ -1030,7 +1027,7 @@ on_get_hosts_cb (gpointer service,
 {
 	EphyHistoryWindow *window = EPHY_HISTORY_WINDOW (user_data);
 	EphyHistoryHost *selected_host;
-	GList *hosts;
+	GList *hosts = NULL;
 
 	if (success != TRUE)
 		goto out;
@@ -1079,7 +1076,7 @@ filter_now (EphyHistoryWindow *editor,
 	if (hosts)
 	{
 		ephy_history_service_find_hosts (editor->priv->history_service,
-						 from, to,
+						 from, to, NULL,
 						(EphyHistoryJobCallback) on_get_hosts_cb, editor);
 	}
 
@@ -1089,7 +1086,7 @@ filter_now (EphyHistoryWindow *editor,
 		ephy_history_service_find_urls (editor->priv->history_service,
 						from, to,
 						0, host ? host->id : 0,
-						substrings,
+						substrings, NULL,
 						(EphyHistoryJobCallback)on_find_urls_cb, editor);
 		ephy_history_host_free (host);
 	}
@@ -1098,6 +1095,7 @@ filter_now (EphyHistoryWindow *editor,
 static gboolean
 on_visit_url_cb (EphyHistoryService *service,
 		 gchar *url,
+		 EphyHistoryPageVisitType visit_type,
 		 EphyHistoryWindow *editor)
 {
 	filter_now (editor, TRUE, TRUE);
@@ -1147,9 +1145,9 @@ ephy_history_window_constructed (GObject *object)
 
 	gtk_ui_manager_insert_action_group (ui_merge,
 					    action_group, 0);
-	gtk_ui_manager_add_ui_from_file (ui_merge,
-					 ephy_file ("epiphany-history-window-ui.xml"),
-					 NULL);
+	gtk_ui_manager_add_ui_from_resource (ui_merge,
+					     "/org/gnome/epiphany/epiphany-history-window-ui.xml",
+					     NULL);
 	gtk_ui_manager_ensure_update (ui_merge);
 	editor->priv->ui_merge = ui_merge;
 	editor->priv->action_group = action_group;
@@ -1270,6 +1268,9 @@ ephy_history_window_constructed (GObject *object)
 			  "row-activated",
 			  G_CALLBACK (ephy_history_window_row_activated_cb),
 			  editor);
+	g_signal_connect (pages_view, "row-middle-clicked",
+			  G_CALLBACK (ephy_history_window_row_middle_clicked_cb),
+			  editor);
 	g_signal_connect (G_OBJECT (pages_view),
 			  "popup_menu",
 			  G_CALLBACK (ephy_history_window_show_popup_cb),
