@@ -22,9 +22,10 @@
 
 #include "config.h"
 
-#include "eggsmclient.h"
 #include "ephy-debug.h"
+#include "ephy-embed-prefs.h"
 #include "ephy-file-helpers.h"
+#include "ephy-private.h"
 #include "ephy-session.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
@@ -333,7 +334,6 @@ main (int argc,
 #endif
 
   g_option_context_add_group (option_context, gtk_get_option_group (TRUE));
-  g_option_context_add_group (option_context, egg_sm_client_get_option_group ());
 
 #ifdef GNOME_ENABLE_DEBUG
   option_group = g_option_group_new ("debug",
@@ -379,6 +379,11 @@ main (int argc,
     exit (1);
   }
 
+  if (application_mode && !g_file_test (profile_directory, G_FILE_TEST_IS_DIR)) {
+      g_print ("--profile must be an existing directory when --application-mode is requested\n");
+      exit (1);
+  }
+
   /* Work-around Flash Player crash */
   g_setenv ("XLIB_SKIP_ARGB_VISUALS", "1", FALSE);
 
@@ -433,10 +438,13 @@ main (int argc,
     mode = EPHY_EMBED_SHELL_MODE_PRIVATE;
   else if (application_mode) {
     char *app_name;
+    char *app_icon;
 
     mode = EPHY_EMBED_SHELL_MODE_APPLICATION;
 
     app_name = g_strrstr (profile_directory, EPHY_WEB_APP_PREFIX);
+    app_icon = g_build_filename (profile_directory, EPHY_WEB_APP_ICON_NAME, NULL);
+
     if (app_name) {
       /* Skip the 'app-' part */
       app_name += strlen (EPHY_WEB_APP_PREFIX);
@@ -444,17 +452,24 @@ main (int argc,
       g_set_prgname (app_name);
       g_set_application_name (app_name);
 
+      gtk_window_set_default_icon_from_file (app_icon, NULL);
+
       /* We need to re-set this because we have already parsed the
        * options, which inits GTK+ and sets this as a side effect. */
       gdk_set_program_class (app_name);
     }
+
+    g_free (app_icon);
   } else {
     mode = EPHY_EMBED_SHELL_MODE_BROWSER;
 
     g_set_prgname ("epiphany");
     g_set_application_name (_("Web"));
+
+    gtk_window_set_default_icon_name ("web-browser");
   }
 
+  ephy_embed_prefs_init ();
   _ephy_shell_create_instance (mode);
 
   startup_flags = get_startup_flags ();
@@ -476,6 +491,7 @@ main (int argc,
 
   ephy_file_save_accels ();
   ephy_state_save ();
+  ephy_embed_prefs_shutdown ();
   ephy_settings_shutdown ();
   ephy_file_helpers_shutdown ();
   xmlCleanupParser ();
