@@ -54,7 +54,6 @@ struct _EphyNotebookPrivate
 	guint tabs_vis_notifier_id;
 
 	guint show_tabs : 1;
-	guint dnd_enabled : 1;
 };
 
 static void ephy_notebook_init		 (EphyNotebook *notebook);
@@ -78,7 +77,6 @@ static const GtkTargetEntry url_drag_types [] =
 enum
 {
 	PROP_0,
-	PROP_DND_ENABLED,
 	PROP_SHOW_TABS
 };
 
@@ -94,18 +92,6 @@ G_DEFINE_TYPE_WITH_CODE (EphyNotebook, ephy_notebook, GTK_TYPE_NOTEBOOK,
 			 G_IMPLEMENT_INTERFACE (EPHY_TYPE_LINK,
 						NULL))
 
-void
-ephy_notebook_set_dnd_enabled (EphyNotebook *notebook,
-			       gboolean enabled)
-{
-	EphyNotebookPrivate *priv = notebook->priv;
-
-	priv->dnd_enabled = enabled;
-	/* FIXME abort any DNDs in progress */
-
-	g_object_notify (G_OBJECT (notebook), "dnd-enabled");
-}
-
 static void
 ephy_notebook_get_property (GObject *object,
 			    guint prop_id,
@@ -117,9 +103,6 @@ ephy_notebook_get_property (GObject *object,
 
 	switch (prop_id)
 	{
-		case PROP_DND_ENABLED:
-			g_value_set_boolean (value, priv->dnd_enabled);
-			break;
 		case PROP_SHOW_TABS:
 			g_value_set_boolean (value, priv->show_tabs);
 			break;
@@ -136,9 +119,6 @@ ephy_notebook_set_property (GObject *object,
 
 	switch (prop_id)
 	{
-		case PROP_DND_ENABLED:
-			ephy_notebook_set_dnd_enabled (notebook, g_value_get_boolean (value));
-			break;
 		case PROP_SHOW_TABS:
 			ephy_notebook_set_show_tabs (notebook, g_value_get_boolean (value));
 			break;
@@ -171,12 +151,6 @@ ephy_notebook_class_init (EphyNotebookClass *klass)
 			      1,
 			      GTK_TYPE_WIDGET /* Can't use an interface type here */);
 
-	g_object_class_install_property (object_class,
-					 PROP_DND_ENABLED,
-					 g_param_spec_boolean ("dnd-enabled", NULL, NULL,
-							       TRUE,
-							       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
-
 	g_object_class_install_property (object_class,
 					 PROP_SHOW_TABS,
 					 g_param_spec_boolean ("show-tabs", NULL, NULL,
@@ -422,22 +396,24 @@ static void
 update_tabs_visibility (EphyNotebook *nb,
 			gboolean before_inserting)
 {
-	EphyNotebookPrivate *priv = nb->priv;
 	EphyEmbedShellMode mode;
-	gboolean show_tabs;
+	gboolean show_tabs = FALSE;
 	guint num;
+	EphyPrefsUITabsBarVisibilityPolicy policy;
 
 	mode = ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (ephy_shell_get_default ()));
 	num = gtk_notebook_get_n_pages (GTK_NOTEBOOK (nb));
 
 	if (before_inserting) num++;
 
-	show_tabs = mode != EPHY_EMBED_SHELL_MODE_APPLICATION &&
-		(g_settings_get_boolean (EPHY_SETTINGS_UI,
-					 EPHY_PREFS_UI_ALWAYS_SHOW_TABS_BAR)
-		 || num > 1) &&
-		priv->show_tabs == TRUE;
+	policy = g_settings_get_enum (EPHY_SETTINGS_UI,
+				      EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY);
 
+	if (mode != EPHY_EMBED_SHELL_MODE_APPLICATION &&
+	    ((policy == EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY_MORE_THAN_ONE && num > 1) ||
+	     policy == EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY_ALWAYS))
+		show_tabs = TRUE;
+	    
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (nb), show_tabs);
 }
 
@@ -464,7 +440,6 @@ ephy_notebook_init (EphyNotebook *notebook)
 	gtk_notebook_set_group_name (gnotebook, EPHY_NOTEBOOK_TAB_GROUP_ID);
 
 	priv->show_tabs = TRUE;
-	priv->dnd_enabled = TRUE;
 
 	g_signal_connect (notebook, "button-press-event",
 			  (GCallback)button_press_cb, NULL);
@@ -482,7 +457,7 @@ ephy_notebook_init (EphyNotebook *notebook)
 	gtk_drag_dest_add_text_targets (widget);
 
 	g_signal_connect (EPHY_SETTINGS_UI,
-			  "changed::" EPHY_PREFS_UI_ALWAYS_SHOW_TABS_BAR,
+			  "changed::" EPHY_PREFS_UI_TABS_BAR_VISIBILITY_POLICY,
 			  G_CALLBACK (show_tabs_changed_cb), notebook);
 }
 
@@ -492,6 +467,9 @@ ephy_notebook_finalize (GObject *object)
 	EphyNotebook *notebook = EPHY_NOTEBOOK (object);
 	EphyNotebookPrivate *priv = notebook->priv;
 
+	g_signal_handlers_disconnect_by_func (EPHY_SETTINGS_UI,
+					      show_tabs_changed_cb,
+					      notebook);
 	g_list_free (priv->focused_pages);
 
 	G_OBJECT_CLASS (ephy_notebook_parent_class)->finalize (object);
@@ -536,12 +514,6 @@ sync_label (EphyWebView *view, GParamSpec *pspec, GtkWidget *label)
 	title = ephy_web_view_get_title (view);
 
 	gtk_label_set_text (GTK_LABEL (label), title);
-
-	/* Set the tooltip on the label's parent (the tab label hbox),
-	 * so it covers all of the tab label.
-	 */
-	gtk_widget_set_tooltip_text (gtk_widget_get_parent (label),
-				     title);
 }
 
 static void
@@ -806,10 +778,67 @@ ephy_notebook_remove (GtkContainer *container,
 	GTK_CONTAINER_CLASS (ephy_notebook_parent_class)->remove (container, tab_widget);
 
 	update_tabs_visibility (notebook, FALSE);
+}
 
-	/* if that was the last tab, destroy the window */
-	if (gtk_notebook_get_n_pages (gnotebook) == 0)
-	{
-		gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (notebook)));
+/**
+ * ephy_notebook_next_page:
+ * @notebook: an #EphyNotebook
+ * 
+ * Advances to the next page in the @notebook. Note that unlike
+ * gtk_notebook_next_page() this method will wrap around if
+ * #GtkSettings:gtk-keynav-wrap-around is set.
+ **/
+void
+ephy_notebook_next_page (EphyNotebook *notebook)
+{
+	gint current_page, n_pages;
+
+	g_return_if_fail (EPHY_IS_NOTEBOOK (notebook));
+
+	current_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+	n_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
+
+	if (current_page < n_pages - 1)
+		gtk_notebook_next_page (GTK_NOTEBOOK (notebook));
+	else {
+		gboolean  wrap_around;
+		
+		g_object_get (gtk_widget_get_settings (GTK_WIDGET (notebook)),
+			      "gtk-keynav-wrap-around", &wrap_around,
+			      NULL);
+		
+		if (wrap_around)
+			gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 0);
+	}
+}
+
+/**
+ * ephy_notebook_prev_page:
+ * @notebook: an #EphyNotebook
+ *
+ * Advances to the previous page in the @notebook. Note that unlike
+ * gtk_notebook_next_page() this method will wrap around if
+ * #GtkSettings:gtk-keynav-wrap-around is set.
+ **/
+void
+ephy_notebook_prev_page (EphyNotebook *notebook)
+{
+	gint current_page;
+
+	g_return_if_fail (EPHY_IS_NOTEBOOK (notebook));
+
+	current_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+
+	if (current_page > 0)
+		gtk_notebook_prev_page (GTK_NOTEBOOK (notebook));
+	else {
+		gboolean  wrap_around;
+		
+		g_object_get (gtk_widget_get_settings (GTK_WIDGET (notebook)),
+			      "gtk-keynav-wrap-around", &wrap_around,
+			      NULL);
+		
+		if (wrap_around)
+			gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), -1);
 	}
 }
