@@ -23,7 +23,6 @@
 #include "ephy-session.h"
 
 #include "eggdesktopfile.h"
-#include "eggsmclient.h"
 #include "ephy-bookmarks-editor.h"
 #include "ephy-debug.h"
 #include "ephy-embed-container.h"
@@ -62,22 +61,17 @@ struct _EphySessionPrivate
 {
 	GList *windows;
 	GList *tool_windows;
-	GtkWidget *resume_infobar;
 	GtkWidget *resume_window;
 
 	GQueue *queue;
 	guint queue_idle_id;
 
-	GtkWidget *quit_interact_dialog;
-
 	guint dont_save : 1;
-	guint quit_while_resuming : 1;
-	guint loading_homepage : 1;
 };
 
 #define BOOKMARKS_EDITOR_ID	"BookmarksEditor"
 #define HISTORY_WINDOW_ID	"HistoryWindow"
-#define SESSION_CRASHED		"type:session_crashed"
+#define SESSION_STATE		"type:session_state"
 
 static void ephy_session_class_init	(EphySessionClass *klass);
 static void ephy_session_iface_init	(EphyExtensionIface *iface);
@@ -94,231 +88,6 @@ G_DEFINE_TYPE_WITH_CODE (EphySession, ephy_session, G_TYPE_OBJECT,
 			 G_IMPLEMENT_INTERFACE (EPHY_TYPE_EXTENSION,
 						ephy_session_iface_init))
 
-/* Gnome session client */
-
-typedef struct
-{
-	EphySession *session;
-	EggSMClient *sm_client;
-	GtkWidget *dialog;
-	GtkWidget *label;
-	guint timeout_id;
-	guint ticks;
-	int response;
-} InteractData;
-
-static void
-confirm_shutdown_dialog_update_timeout_label (InteractData *data)
-{
-	char *text;
-
-	text = g_strdup_printf (ngettext ("Downloads will be aborted and logout proceed in %d second.",
-					  "Downloads will be aborted and logout proceed in %d seconds.",
-					  data->ticks),
-				data->ticks);
-
-	gtk_label_set_text (GTK_LABEL (data->label), text);
-	g_free (text);
-}
-		
-static gboolean
-confirm_shutdown_dialog_tick_cb (InteractData *data)
-{
-	if (data->ticks > 0)
-	{
-		--data->ticks;
-		confirm_shutdown_dialog_update_timeout_label (data);
-		return TRUE;
-	}
-
-	data->timeout_id = 0;
-	gtk_dialog_response (GTK_DIALOG (data->dialog),
-			     GTK_RESPONSE_ACCEPT);
-	return FALSE;
-}
-
-static void
-confirm_shutdown_dialog_response_cb (GtkWidget *dialog,
-				     int response,
-				     InteractData *data)
-{
-	LOG ("confirm_shutdown_dialog_response_cb response %d", response);
-
-	data->response = response;
-
-	gtk_widget_destroy (dialog);
-}
-
-static void
-confirm_shutdown_dialog_accept_cb (InteractData *data,
-				   GObject *zombie)
-{
-	gtk_dialog_response (GTK_DIALOG (data->dialog),
-			     GTK_RESPONSE_ACCEPT);
-}
-
-static void
-confirm_shutdown_dialog_weak_ref_cb (InteractData *data,
-				     GObject *zombie)
-{
-	EphySessionPrivate *priv = data->session->priv;
-	EggSMClient *sm_client = data->sm_client;
-	EphyShell *shell;
-	gboolean will_quit;
-
-	LOG ("confirm_shutdown_dialog_weak_ref_cb response %d", data->response);
-
-	priv->quit_interact_dialog = NULL;
-
-	shell = ephy_shell_get_default ();
-	if (shell != NULL)
-	{
-		g_object_weak_unref (G_OBJECT (shell),
-				     (GWeakNotify) confirm_shutdown_dialog_accept_cb,
-				     data);
-	}
-
-	if (data->timeout_id != 0)
-	{
-		g_source_remove (data->timeout_id);
-	}
-
-	will_quit = data->response == GTK_RESPONSE_ACCEPT;
-
-	g_free (data);
-
-	egg_sm_client_will_quit (sm_client, will_quit);
-	g_object_unref (sm_client);
-}
-
-static void
-client_quit_requested_cb (EggSMClient *sm_client,
-			  EphySession *session)
-{
-	EphySessionPrivate *priv = session->priv;
-	GtkWidget *dialog, *box;
-	InteractData *data;
-	GList *downloads;
-
-	/* If we're shutting down, check if there are downloads
-	 * remaining, since they can't be restarted.
-	 */
-
-	downloads = ephy_embed_shell_get_downloads (embed_shell);
-	if (ephy_shell_get_default () == NULL || downloads == NULL)
-	{
-		egg_sm_client_will_quit (sm_client, TRUE);
-		return;
-	}
-
-	dialog = gtk_message_dialog_new
-		(NULL,
-		 GTK_DIALOG_MODAL,
-		 GTK_MESSAGE_WARNING,
-		 GTK_BUTTONS_NONE,
-		 _("Abort pending downloads?"));
-	priv->quit_interact_dialog = dialog;
-
-	gtk_message_dialog_format_secondary_text
-		(GTK_MESSAGE_DIALOG (dialog),
-		 _("There are still downloads pending. If you log out, "
-		   "they will be aborted and lost."));
-
-	gtk_dialog_add_button (GTK_DIALOG (dialog),
-			       _("_Cancel Logout"), GTK_RESPONSE_REJECT);
-	gtk_dialog_add_button (GTK_DIALOG (dialog),
-			       _("_Abort Downloads"), GTK_RESPONSE_ACCEPT);
-
-	gtk_window_set_title (GTK_WINDOW (dialog), "");
-	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_REJECT);
-
-	data = g_new (InteractData, 1);
-	data->sm_client = g_object_ref (sm_client);
-	data->session = session;
-	data->dialog = dialog;
-	data->response = GTK_RESPONSE_REJECT;
-
-	/* This isn't very exact, but it's good enough here */
-	data->timeout_id = g_timeout_add_seconds (1,
-					  (GSourceFunc) confirm_shutdown_dialog_tick_cb,
-					  data);
-	data->ticks = 60;
-
-	/* Add timeout label */
-	data->label = gtk_label_new (NULL);
-	gtk_label_set_line_wrap (GTK_LABEL (data->label), TRUE);
-	confirm_shutdown_dialog_update_timeout_label (data);
-
-	box = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
-	gtk_box_pack_end (GTK_BOX (box), data->label, FALSE, FALSE, 0);
-	gtk_widget_show (data->label);
-
-	/* When we're quitting, un-veto the shutdown  */
-	g_object_weak_ref (G_OBJECT (ephy_shell_get_default ()),
-			   (GWeakNotify) confirm_shutdown_dialog_accept_cb,
-			   data);
-
-	g_signal_connect (dialog, "response",
-			  G_CALLBACK (confirm_shutdown_dialog_response_cb), data);
-	g_object_weak_ref (G_OBJECT (dialog),
-			   (GWeakNotify) confirm_shutdown_dialog_weak_ref_cb,
-			   data);
-
-	gtk_window_present (GTK_WINDOW (dialog));
-}
-
-static void
-client_quit_cancelled_cb (EggSMClient *sm_client,
-			  EphySession *session)
-{
-	EphySessionPrivate *priv = session->priv;
-
-	if (priv->quit_interact_dialog)
-	{
-		gtk_dialog_response (GTK_DIALOG (priv->quit_interact_dialog),
-				     GTK_RESPONSE_DELETE_EVENT);
-	}
-}
-
-static void
-client_quit_cb (EggSMClient *sm_client,
-		EphySession *session)
-{
-	LOG ("quit-cb");
-
-	ephy_session_close (session);
-}
-
-static void
-client_save_state_cb (EggSMClient *sm_client,
-		      GKeyFile *keyfile,
-		      EphySession *session)
-{
-	char *argv[] = { NULL, "--load-session", NULL };
-	char *discard_argv[] = { "rm", "-f", NULL };
-	char *tmp, *save_to;
-
-	LOG ("save_yourself_cb");
-
-	tmp = g_build_filename (ephy_dot_dir (),
-				"session_gnome-XXXXXX",
-				NULL);
-	save_to = ephy_file_tmp_filename (tmp, "xml");
-	g_free (tmp);
-
-	argv[0] = g_get_prgname ();
-	argv[2] = save_to;
-	egg_sm_client_set_restart_command (sm_client, 3, (const char **) argv);
-
-	discard_argv[2] = save_to;
-	egg_sm_client_set_discard_command (sm_client, 3, (const char **) discard_argv);
-
-	ephy_session_save (session, save_to);
-
-	g_free (save_to);
-}
-
 /* Helper functions */
 
 static GFile *
@@ -332,10 +101,10 @@ get_session_file (const char *filename)
 		return NULL;
 	}
 
-	if (strcmp (filename, SESSION_CRASHED) == 0)
+	if (strcmp (filename, SESSION_STATE) == 0)
 	{
 		path = g_build_filename (ephy_dot_dir (),
-					 "session_crashed.xml",
+					 "session_state.xml",
 					 NULL);
 	}
 	else
@@ -373,7 +142,7 @@ load_status_notify_cb (EphyWebView *view,
 	if (status == WEBKIT_LOAD_PROVISIONAL ||
 	    status == WEBKIT_LOAD_COMMITTED || 
 	    status == WEBKIT_LOAD_FINISHED)
-		ephy_session_save (session, SESSION_CRASHED);
+		ephy_session_save (session, SESSION_STATE);
 }
 
 static void
@@ -392,7 +161,7 @@ notebook_page_removed_cb (GtkWidget *notebook,
 			  guint position,
 			  EphySession *session)
 {
-	ephy_session_save (session, SESSION_CRASHED);
+	ephy_session_save (session, SESSION_STATE);
 
 	g_signal_handlers_disconnect_by_func
 		(ephy_embed_get_web_view (embed), G_CALLBACK (load_status_notify_cb),
@@ -405,7 +174,7 @@ notebook_page_reordered_cb (GtkWidget *notebook,
 			    guint position,
 			    EphySession *session)
 {
-	ephy_session_save (session, SESSION_CRASHED);
+	ephy_session_save (session, SESSION_STATE);
 }
 
 static gboolean
@@ -440,7 +209,7 @@ session_command_free (SessionCommand *cmd)
 		g_strfreev (cmd->args);
 	}
 
-	g_free (cmd);
+	g_slice_free (SessionCommand, cmd);
 
 	g_object_unref (ephy_shell_get_default ());
 }
@@ -454,46 +223,6 @@ session_command_find (const SessionCommand *cmd,
 	return command != cmd->command;
 }
 
-static void
-resume_infobar_response_cb (GtkWidget *info_bar,
-			    int response,
-			    EphySession *session)
-{
-	guint32 user_time;
-	EphySessionPrivate *priv = session->priv;
-
-	LOG ("resume_infobar_response_cb response:%d", response);
-
-	gtk_widget_hide (info_bar);
-
-	user_time = gtk_get_current_event_time ();
-
-	priv->dont_save = FALSE;
-
-	gtk_widget_destroy (info_bar);
-	priv->resume_infobar = NULL;
-
-	if (response == GTK_RESPONSE_YES)
-	{
-		ephy_session_queue_command (session,
-					    EPHY_SESSION_CMD_LOAD_SESSION,
-					    SESSION_CRASHED, NULL,
-					    user_time, TRUE);
-	}
-}
-
-static void
-resume_infobar_weak_ref_cb (EphySession *session,
-			    GObject *zombie)
-{
-	EphySessionPrivate *priv = session->priv;
-
-	LOG ("resume_window_infobar_weak_ref_cb");
-
-	priv->dont_save = FALSE;
-	session_command_queue_next (session);
-}
-
 static void
 session_command_autoresume (EphySession *session,
 			    guint32 user_time)
@@ -502,26 +231,30 @@ session_command_autoresume (EphySession *session,
 	GFile *saved_session_file;
 	char *saved_session_file_path;
 	gboolean crashed_session;
+	EphyPrefsRestoreSessionPolicy policy;
 
 	LOG ("ephy_session_autoresume");
 
-	saved_session_file = get_session_file (SESSION_CRASHED);
+	saved_session_file = get_session_file (SESSION_STATE);
 	saved_session_file_path = g_file_get_path (saved_session_file);
 	g_object_unref (saved_session_file);
 	crashed_session = g_file_test (saved_session_file_path, G_FILE_TEST_EXISTS);
 	
 	g_free (saved_session_file_path);
 
+	policy = g_settings_get_enum (EPHY_SETTINGS_MAIN,
+				      EPHY_PREFS_RESTORE_SESSION_POLICY);
+
 	if (crashed_session == FALSE ||
+	    policy == EPHY_PREFS_RESTORE_SESSION_POLICY_NEVER ||
 	    priv->windows != NULL ||
 	    priv->tool_windows != NULL)
 	{
-		/* FIXME can this happen? */
-		if (priv->resume_infobar != NULL)
-		{
-			gtk_widget_hide (priv->resume_infobar);
-			gtk_widget_destroy (priv->resume_infobar);
-		}
+		/* If we are auto-resuming, and we never want to
+		 * restore the session, clobber the session state
+		 * file. */
+		if (policy == EPHY_PREFS_RESTORE_SESSION_POLICY_NEVER)
+			session_delete (session, SESSION_STATE);
 
 		ephy_session_queue_command (session,
 					    EPHY_SESSION_CMD_MAYBE_OPEN_WINDOW,
@@ -539,8 +272,8 @@ session_command_autoresume (EphySession *session,
 	}
 
 	ephy_session_queue_command (session,
-				    EPHY_SESSION_CMD_MAYBE_OPEN_WINDOW_RESTORE,
-				    NULL, NULL, user_time, TRUE);
+				    EPHY_SESSION_CMD_LOAD_SESSION,
+				    SESSION_STATE, NULL, user_time, TRUE);
 }
 
 static void
@@ -613,24 +346,11 @@ session_command_open_uris (EphySession *session,
 		 * that case use a new tab in the same window */
 		if (i == 0 && priv->resume_window != NULL)
 		{
-			if (priv->resume_infobar != NULL)
-			{
-				embed = ephy_shell_new_tab_full (shell, EPHY_WINDOW (priv->resume_window),
-								 NULL /* parent tab */,
-								 request,
-								 EPHY_NEW_TAB_IN_EXISTING_WINDOW | page_flags,
-								 EPHY_WEB_VIEW_CHROME_ALL,
-								 FALSE /* is popup? */,
-								 user_time);
-			}
-			else
-			{
-				EphyWebView *web_view;
-
-				embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (priv->resume_window));
-				web_view = ephy_embed_get_web_view (embed);
-				ephy_web_view_load_url (web_view, url);
-			}
+			EphyWebView *web_view;
+			
+			embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (priv->resume_window));
+			web_view = ephy_embed_get_web_view (embed);
+			ephy_web_view_load_url (web_view, url);
 		}
 		else
 		{
@@ -652,46 +372,6 @@ session_command_open_uris (EphySession *session,
 	g_object_unref (shell);
 }
 
-static void
-send_no_response_cb (GtkButton *button, GtkInfoBar *info_bar)
-{
-	gtk_info_bar_response (info_bar, GTK_RESPONSE_NO);
-}
-
-static void
-send_yes_response_cb (GtkButton *button, GtkInfoBar *info_bar)
-{
-	gtk_info_bar_response (info_bar, GTK_RESPONSE_YES);
-}
-
-static void
-loading_homepage_cb (EphyWebView *view, EphySession *session)
-{
-	EphySessionPrivate *priv = session->priv;
-
-	priv->loading_homepage = TRUE;
-}
-
-static void
-new_document_now_cb (EphyWebView *view, const char *uri, EphySession *session)
-{
-	EphySessionPrivate *priv = session->priv;
-
-	if (priv->loading_homepage)
-	{
-		priv->loading_homepage = FALSE;
-		return;
-	}
-
-	if (priv->resume_infobar) 
-	{
-		EphyEmbed *embed = EPHY_GET_EMBED_FROM_EPHY_WEB_VIEW (view);
-		ephy_embed_remove_top_widget (embed, priv->resume_infobar);
-		priv->resume_infobar = NULL;
-		priv->resume_window = NULL;
-	}
-}
-
 static gboolean
 session_command_dispatch (EphySession *session)
 {
@@ -733,74 +413,6 @@ session_command_dispatch (EphySession *session)
 							 cmd->user_time);
 			}
 			break;
-		case EPHY_SESSION_CMD_MAYBE_OPEN_WINDOW_RESTORE:
-			/* FIXME: maybe just check for normal windows? */
-			if (priv->windows == NULL &&
-			    priv->tool_windows == NULL)
-			{
-				GtkWidget *info_bar;
-				GtkWidget *action_area;
-				GtkWidget *button_box;
-				GtkWidget *action_button;
-				GtkWidget *content_area;
-				GtkWidget *label;
-				EphyEmbed *embed;
-				EphyWebView *view;
-
-				session->priv->dont_save = TRUE;
-
-				embed = ephy_shell_new_tab_full (ephy_shell_get_default (),
-								 NULL /* window */, NULL /* tab */,
-								 NULL /* Networ	kRequest */,
-								 EPHY_NEW_TAB_IN_NEW_WINDOW |
-								 EPHY_NEW_TAB_HOME_PAGE,
-								 EPHY_WEB_VIEW_CHROME_ALL,
-								 FALSE /* is popup? */,
-								 cmd->user_time);
-
-				info_bar = gtk_info_bar_new ();
-
-				session->priv->resume_infobar = info_bar;
-				session->priv->resume_window = gtk_widget_get_toplevel (GTK_WIDGET (embed));
-
-				action_area = gtk_info_bar_get_action_area (GTK_INFO_BAR (info_bar));
-				button_box = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
-				gtk_container_add (GTK_CONTAINER (action_area), button_box);
-
-				action_button = gtk_button_new_with_mnemonic (_("_Don't recover"));
-				g_signal_connect (action_button, "clicked",
-						  G_CALLBACK (send_no_response_cb), info_bar);
-				gtk_box_pack_start (GTK_BOX (button_box), action_button, FALSE, FALSE, 0);
-
-				action_button = gtk_button_new_with_mnemonic (_("_Recover session"));
-				g_signal_connect (action_button, "clicked",
-						  G_CALLBACK (send_yes_response_cb), info_bar);
-				gtk_box_pack_start (GTK_BOX (button_box), action_button, FALSE, FALSE, 0);
-
-				label = gtk_label_new (_("Do you want to recover the previous browser windows and tabs?"));
-				gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-				content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (info_bar));
-				gtk_container_add (GTK_CONTAINER (content_area), label);
-				gtk_widget_show_all (info_bar);
-
-				g_signal_connect (info_bar, "response",
-						  G_CALLBACK (resume_infobar_response_cb), session);
-
-				g_object_weak_ref (G_OBJECT (info_bar),
-						   (GWeakNotify) resume_infobar_weak_ref_cb,
-						   session);
-				
-				ephy_embed_add_top_widget (embed, info_bar, FALSE);
-
-				view = ephy_embed_get_web_view (embed);
-
-				g_signal_connect (view, "loading-homepage",
-						  G_CALLBACK (loading_homepage_cb), session);
-
-				g_signal_connect (view, "new-document-now",
-						  G_CALLBACK (new_document_now_cb), session);
-			}
-			break;
 		default:
 			g_assert_not_reached ();
 			break;
@@ -829,7 +441,6 @@ session_command_queue_next (EphySession *session)
 	LOG ("queue_next");
 
 	if (!g_queue_is_empty (priv->queue) &&
-	    priv->resume_infobar == NULL &&
 	    priv->queue_idle_id == 0)
 	{
 		priv->queue_idle_id =
@@ -869,7 +480,7 @@ impl_attach_window (EphyExtension *extension,
 	LOG ("impl_attach_window");
 
 	session->priv->windows = g_list_append (session->priv->windows, window);
-	ephy_session_save (session, SESSION_CRASHED);
+	ephy_session_save (session, SESSION_STATE);
 
 	g_signal_connect (window, "focus-in-event",
 			  G_CALLBACK (window_focus_in_event_cb), session);
@@ -908,7 +519,7 @@ impl_detach_window (EphyExtension *extension,
 	LOG ("impl_detach_window");
 
 	session->priv->windows = g_list_remove (session->priv->windows, window);
-	ephy_session_save (session, SESSION_CRASHED);
+	ephy_session_save (session, SESSION_STATE);
 
 	/* NOTE: since the window will be destroyed anyway, we don't need to
 	 * disconnect our signal handlers from its components.
@@ -921,48 +532,23 @@ static void
 ephy_session_init (EphySession *session)
 {
 	EphySessionPrivate *priv;
-	EggSMClient *sm_client;
 
 	LOG ("EphySession initialising");
 
 	priv = session->priv = EPHY_SESSION_GET_PRIVATE (session);
 
 	priv->queue = g_queue_new ();
-
-	sm_client = egg_sm_client_get ();
-	g_signal_connect (sm_client, "save-state",
-			  G_CALLBACK (client_save_state_cb), session);
-	g_signal_connect (sm_client, "quit-requested",
-			  G_CALLBACK (client_quit_requested_cb), session);
-	g_signal_connect (sm_client, "quit-cancelled",
-			  G_CALLBACK (client_quit_cancelled_cb), session);
-	g_signal_connect (sm_client, "quit",
-			  G_CALLBACK (client_quit_cb), session);
 }
 
 static void
 ephy_session_dispose (GObject *object)
 {
 	EphySession *session = EPHY_SESSION (object);
-	EphySessionPrivate *priv = session->priv;
-	EggSMClient *sm_client;
 
 	LOG ("EphySession disposing");
 
-	/* Only remove the crashed session if we're not shutting down while
-	 * the session resume dialogue was still shown!
-	*/
-	if (priv->quit_while_resuming == FALSE)
-	{
-		session_delete (session, SESSION_CRASHED);
-	}
-
 	session_command_queue_clear (session);
 
-	sm_client = egg_sm_client_get ();
-	g_signal_handlers_disconnect_matched (sm_client, G_SIGNAL_MATCH_DATA,
-					      0, 0, NULL, NULL, session);
-
 	G_OBJECT_CLASS (ephy_session_parent_class)->dispose (object);
 }
 
@@ -1039,69 +625,26 @@ ephy_session_class_init (EphySessionClass *class)
 
 /* Implementation */
 
-static void
-close_dialog (GtkWidget *widget)
-{
-	if (GTK_IS_DIALOG (widget))
-	{
-		/* don't destroy them, someone might have a ref on them */
-		gtk_dialog_response (GTK_DIALOG (widget),
-				     GTK_RESPONSE_DELETE_EVENT);
-	}
-}
-
 void
 ephy_session_close (EphySession *session)
 {
-	EphySessionPrivate *priv = session->priv;
-	GList *windows;
+	EphyPrefsRestoreSessionPolicy policy;
 
 	LOG ("ephy_session_close");
 
-	/* we have to ref the shell or else we may get finalised between
-	 * destroying the windows and destroying the tool windows
-	 */
-	g_object_ref (ephy_shell_get_default ());
+	policy = g_settings_get_enum (EPHY_SETTINGS_MAIN,
+				      EPHY_PREFS_RESTORE_SESSION_POLICY);
+	if (policy == EPHY_PREFS_RESTORE_SESSION_POLICY_ALWAYS)
+	{
+		EphySessionPrivate *priv = session->priv;
 
-	priv->dont_save = TRUE;
-	/* need to set this up here while the dialogue hasn't been killed yet */
-	priv->quit_while_resuming = priv->resume_window != NULL;	
+		priv->dont_save = TRUE;
 
-	/* Clear command queue */
-	session_command_queue_clear (session);
+		session_command_queue_clear (session);
 
-	ephy_embed_shell_prepare_close (embed_shell);
+		ephy_embed_shell_prepare_close (embed_shell);
 
-	/* there may still be windows open, like dialogues posed from
-	* web pages, etc. Try to kill them, but be sure NOT to destroy
-	* the gtkmozembed offscreen window!
-	* Here, we just check if it's a dialogue and close it if it is one.
-	*/
-	windows = gtk_window_list_toplevels ();
-	g_list_foreach (windows, (GFunc) close_dialog, NULL);
-	g_list_free (windows);
-
-	windows	= ephy_session_get_windows (session);
-	g_list_foreach (windows, (GFunc) gtk_widget_destroy, NULL);
-	g_list_free (windows);
-
-	windows = g_list_copy (session->priv->tool_windows);
-	g_list_foreach (windows, (GFunc) gtk_widget_destroy, NULL);
-	g_list_free (windows);	
-
-	ephy_embed_shell_prepare_close (embed_shell);
-
-	/* Just to be really sure, do it again: */
-	windows = gtk_window_list_toplevels ();
-	g_list_foreach (windows, (GFunc) close_dialog, NULL);
-	g_list_free (windows);
-
-	session->priv->dont_save = FALSE;
-
-	/* Clear command queue */
-	session_command_queue_clear (session);
-
-	g_object_unref (ephy_shell_get_default ());
+	}
 }
 
 static int
@@ -1509,7 +1052,7 @@ restore_geometry (GtkWindow *window,
 	}
 }
 
-/*
+/**
  * ephy_session_load:
  * @session: a #EphySession
  * @filename: the path of the source file
@@ -1641,7 +1184,7 @@ ephy_session_load (EphySession *session,
 	priv->dont_save = FALSE;
 	priv->resume_window = NULL;
 
-	ephy_session_save (session, SESSION_CRASHED);
+	ephy_session_save (session, SESSION_STATE);
 
 	g_object_unref (ephy_shell_get_default ());
 
@@ -1681,7 +1224,7 @@ ephy_session_add_window (EphySession *session,
 		g_list_append (session->priv->tool_windows, window);
 	gtk_application_add_window (GTK_APPLICATION (ephy_shell_get_default ()), window);
 
-	ephy_session_save (session, SESSION_CRASHED);
+	ephy_session_save (session, SESSION_STATE);
 }
 
 /**
@@ -1702,7 +1245,7 @@ ephy_session_remove_window (EphySession *session,
 		g_list_remove (session->priv->tool_windows, window);
 	gtk_application_remove_window (GTK_APPLICATION (ephy_shell_get_default ()), window);
 
-	ephy_session_save (session, SESSION_CRASHED);
+	ephy_session_save (session, SESSION_STATE);
 }
 
 /**
@@ -1787,8 +1330,7 @@ ephy_session_queue_command (EphySession *session,
 		}
 	}
 
-	/* FIXME: use g_slice_new */
-	cmd = g_new0 (SessionCommand, 1);
+	cmd = g_slice_new0 (SessionCommand);
 	cmd->command = command;
 	cmd->arg = arg ? g_strdup (arg) : NULL;
 	cmd->args = args ? g_strdupv ((gchar **)args) : NULL;
