@@ -30,13 +30,13 @@
 #include "ephy-embed-single.h"
 #include "ephy-embed-utils.h"
 #include "ephy-extensions-manager.h"
-#include "ephy-favicon-cache.h"
 #include "ephy-file-helpers.h"
 #include "ephy-gui.h"
 #include "ephy-history-window.h"
 #include "ephy-home-action.h"
 #include "ephy-lockdown.h"
 #include "ephy-prefs.h"
+#include "ephy-private.h"
 #include "ephy-profile-utils.h"
 #include "ephy-session.h"
 #include "ephy-settings.h"
@@ -234,30 +234,15 @@ show_about (GSimpleAction *action,
   window_cmd_help_about (NULL, GTK_WIDGET (window));
 }
 
-static void
-show_help (GSimpleAction *action,
-           GVariant *parameter,
-           gpointer user_data)
-{
-  ephy_gui_help (NULL, NULL);
-}
-
 static void
 quit_application (GSimpleAction *action,
                   GVariant *parameter,
                   gpointer user_data)
 {
-  GList *list, *next;
-  GtkWindow *win;
-
-  list = gtk_application_get_windows (GTK_APPLICATION (g_application_get_default ()));
-  while (list) {
-    win = list->data;
-    next = list->next;
-
-    gtk_widget_destroy (GTK_WIDGET (win));
-
-    list = next;
+  if (!g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
+                               EPHY_PREFS_LOCKDOWN_QUIT)) {
+    ephy_session_close (EPHY_SESSION (ephy_shell_get_session (ephy_shell)));
+    g_application_quit (g_application_get_default ());
   }
 }
 
@@ -267,7 +252,6 @@ static GActionEntry app_entries[] = {
   { "history", show_history, NULL, NULL, NULL },
   { "preferences", show_preferences, NULL, NULL, NULL },
   { "pdm", show_pdm, NULL, NULL, NULL },
-  { "help", show_help, NULL, NULL, NULL },
   { "about", show_about, NULL, NULL, NULL },
   { "quit", quit_application, NULL, NULL, NULL },
 };
@@ -309,9 +293,9 @@ ephy_shell_startup (GApplication* application)
                                      application);
 
     builder = gtk_builder_new ();
-    gtk_builder_add_from_file (builder,
-                               ephy_file ("epiphany-application-menu.ui"),
-                               NULL);
+    gtk_builder_add_from_resource (builder,
+                                   "/org/gnome/epiphany/epiphany-application-menu.ui",
+                                   NULL);
     gtk_application_set_app_menu (GTK_APPLICATION (application),
                                   G_MENU_MODEL (gtk_builder_get_object (builder, "app-menu")));
     g_object_unref (builder);
@@ -561,21 +545,6 @@ ephy_shell_new_window_cb (EphyEmbedSingle *single,
      NULL, NULL, flags, chromemask, is_popup, 0);
 }
 
-static void
-ephy_shell_sync_network_status (GNetworkMonitor *monitor,
-                                gboolean available,
-                                EphyShell *shell)
-{
-  EphyShellPrivate *priv = shell->priv;
-  EphyEmbedSingle *single;
-
-  if (!priv->embed_single_connected) return;
-
-  single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (EPHY_EMBED_SHELL (shell)));
-
-  ephy_embed_single_set_network_status (single, available);
-}
-
 static GObject*
 impl_get_embed_single (EphyEmbedShell *embed_shell)
 {
@@ -592,13 +561,6 @@ impl_get_embed_single (EphyEmbedShell *embed_shell)
                              shell, G_CONNECT_AFTER);
 
     priv->embed_single_connected = TRUE;
-
-    /* Now we need the net monitor   */
-    if (ephy_shell_get_net_monitor (shell)) {
-        ephy_shell_sync_network_status (priv->network_monitor,
-                                        g_network_monitor_get_network_available (priv->network_monitor),
-                                        shell);
-    }
   }
 
   return embed_single;
@@ -614,7 +576,7 @@ ephy_shell_init (EphyShell *shell)
   /* globally accessible singleton */
   g_assert (ephy_shell == NULL);
   ephy_shell = shell;
-  g_object_add_weak_pointer (G_OBJECT(ephy_shell),
+  g_object_add_weak_pointer (G_OBJECT (ephy_shell),
                              (gpointer *)ptr);
 }
 
@@ -677,8 +639,6 @@ ephy_shell_dispose (GObject *object)
 
   if (priv->network_monitor != NULL) {
     LOG ("Unref net monitor ");
-    g_signal_handlers_disconnect_by_func
-      (priv->network_monitor, G_CALLBACK (ephy_shell_sync_network_status), shell);
     g_object_unref (priv->network_monitor);
     priv->network_monitor = NULL;
   }
@@ -837,7 +797,7 @@ ephy_shell_new_tab_full (EphyShell *shell,
     if (is_empty) {
       /* empty page, focus location entry */
       ephy_window_activate_location (window);
-    } else if (embed != NULL) {
+    } else if ((flags & EPHY_NEW_TAB_DONT_SHOW_WINDOW) == 0 && embed != NULL) {
       /* non-empty page, focus the page. but make sure the widget is realised first! */
       gtk_widget_realize (GTK_WIDGET (embed));
       gtk_widget_grab_focus (GTK_WIDGET (embed));
@@ -983,11 +943,8 @@ ephy_shell_get_net_monitor (EphyShell *shell)
 {
   EphyShellPrivate *priv = shell->priv;
 
-  if (priv->network_monitor == NULL) {
+  if (priv->network_monitor == NULL)
     priv->network_monitor = g_network_monitor_get_default ();
-    g_signal_connect (priv->network_monitor, "network-changed",
-                      G_CALLBACK (ephy_shell_sync_network_status), shell);
-  }
 
   return G_OBJECT (priv->network_monitor);
 }
@@ -1040,13 +997,12 @@ ephy_shell_get_bookmarks_editor (EphyShell *shell)
 GtkWidget *
 ephy_shell_get_history_window (EphyShell *shell)
 {
-  EphyHistory *history;
+  EphyHistoryService *service;
 
   if (shell->priv->history_window == NULL) {
-    history = EPHY_HISTORY
-      (ephy_embed_shell_get_global_history (embed_shell));
-    g_assert (history != NULL);
-    shell->priv->history_window = ephy_history_window_new (history);
+    service = EPHY_HISTORY_SERVICE
+      (ephy_embed_shell_get_global_history_service (embed_shell));
+    shell->priv->history_window = ephy_history_window_new (service);
 
     g_signal_connect (shell->priv->history_window, "show",
                       G_CALLBACK (toolwindow_show_cb), shell);
@@ -1073,7 +1029,7 @@ ephy_shell_get_pdm_dialog (EphyShell *shell)
     dialog = &shell->priv->pdm_dialog;
 
     g_object_add_weak_pointer (shell->priv->pdm_dialog,
-                               (gpointer *) dialog);
+                               (gpointer *)dialog);
   }
 
   return shell->priv->pdm_dialog;
@@ -1095,7 +1051,7 @@ ephy_shell_get_prefs_dialog (EphyShell *shell)
     dialog  = &shell->priv->prefs_dialog;
 
     g_object_add_weak_pointer (shell->priv->prefs_dialog,
-                               (gpointer *) dialog);
+                               (gpointer *)dialog);
   }
 
   return shell->priv->prefs_dialog;
