@@ -33,7 +33,6 @@
 #include "ephy-debug.h"
 #include "ephy-time-helpers.h"
 #include "ephy-embed-single.h"
-#include "ephy-favicon-cache.h"
 #include "ephy-history-service.h"
 #include "ephy-password-info.h"
 
@@ -256,7 +255,7 @@ clear_all_dialog_response_cb (GtkDialog *dialog,
 
 			shell = ephy_embed_shell_get_default ();
 			history = EPHY_HISTORY_SERVICE (ephy_embed_shell_get_global_history_service (shell));
-			ephy_history_service_clear (history, NULL, NULL);
+			ephy_history_service_clear (history, NULL, NULL, NULL);
 		}
 		if (gtk_toggle_button_get_active
 			(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_cookies)))
@@ -284,7 +283,6 @@ clear_all_dialog_response_cb (GtkDialog *dialog,
 		{
 			EphyEmbedShell *shell;
 			EphyEmbedSingle *single;
-			EphyFaviconCache *cache;
 
 			shell = ephy_embed_shell_get_default ();
 
@@ -292,8 +290,7 @@ clear_all_dialog_response_cb (GtkDialog *dialog,
 
 			ephy_embed_single_clear_cache (single);
 
-			cache = EPHY_FAVICON_CACHE (ephy_embed_shell_get_favicon_cache (shell));
-			ephy_favicon_cache_clear (cache);
+			webkit_favicon_database_clear (webkit_get_favicon_database ());
 		}
 	}
 	gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -628,7 +625,7 @@ show_cookies_properties (PdmDialog *dialog,
 
 	cookie_dialog = ephy_dialog_new_with_parent (parent);
 	ephy_dialog_construct (cookie_dialog,
-			       ephy_file ("epiphany.ui"),
+			       "/org/gnome/epiphany/epiphany.ui",
 			       "cookie_properties_dialog",
 			       NULL);
 
@@ -1469,7 +1466,7 @@ pdm_dialog_init (PdmDialog *dialog)
 	priv = dialog->priv = EPHY_PDM_DIALOG_GET_PRIVATE (dialog);
 
 	ephy_dialog_construct (EPHY_DIALOG (dialog),
-			       ephy_file ("epiphany.ui"),
+			       "/org/gnome/epiphany/epiphany.ui",
 			       "pdm_dialog",
 			       NULL);
 
