@@ -500,38 +500,32 @@ dialog_save_as_application_response_cb (GtkDialog *dialog,
 					gint response,
 					EphyApplicationDialogData *data)
 {
-	char *profile_dir;
+	const char *app_name;
 	char *desktop_file;
 	char *message;
 	NotifyNotification *notification;
-	gboolean profile_exists;
 
 	if (response == GTK_RESPONSE_OK) {
-		profile_dir = ephy_web_application_get_profile_directory (gtk_entry_get_text (GTK_ENTRY (data->entry)));
-		if (!profile_dir)
-			return;
-		profile_exists = g_file_test (profile_dir, G_FILE_TEST_IS_DIR);
-		g_free (profile_dir);
+		app_name = gtk_entry_get_text (GTK_ENTRY (data->entry));
 
-		if (profile_exists)
+		if (ephy_web_application_exists (app_name))
 		{
-			if (confirm_web_application_overwrite (GTK_WINDOW (dialog),
-							       gtk_entry_get_text (GTK_ENTRY (data->entry))))
-				ephy_web_application_delete (gtk_entry_get_text (GTK_ENTRY (data->entry)));
+			if (confirm_web_application_overwrite (GTK_WINDOW (dialog), app_name))
+				ephy_web_application_delete (app_name);
 			else
 				return;
 		}
 
 		/* Create Web Application, including a new profile and .desktop file. */
-		desktop_file = ephy_web_application_create (data->view,
-							    gtk_entry_get_text (GTK_ENTRY (data->entry)),
+		desktop_file = ephy_web_application_create (webkit_web_view_get_uri (WEBKIT_WEB_VIEW (data->view)),
+							    app_name,
 							    gtk_image_get_pixbuf (GTK_IMAGE (data->image)));
 		if (desktop_file)
 			message = g_strdup_printf (_("The application '%s' is ready to be used"),
-						   gtk_entry_get_text (GTK_ENTRY (data->entry)));
+						   app_name);
 		else
 			message = g_strdup_printf (_("The application '%s' could not be created"),
-						   gtk_entry_get_text (GTK_ENTRY (data->entry)));
+						   app_name);
 
 		notification = notify_notification_new (message,
 							NULL, NULL);
@@ -1250,26 +1244,24 @@ void
 window_cmd_tabs_next (GtkAction *action,
 		      EphyWindow *window)
 {
-	GtkNotebook *nb;
-	gboolean handled;
+	GtkWidget *nb;
 
-	nb = GTK_NOTEBOOK (ephy_window_get_notebook (window));
+	nb = ephy_window_get_notebook (window);
 	g_return_if_fail (nb != NULL);
 
-	g_signal_emit_by_name (nb, "change-current-page", 1, &handled);
+	ephy_notebook_next_page (EPHY_NOTEBOOK (nb));
 }
 
 void
 window_cmd_tabs_previous (GtkAction *action,
 			  EphyWindow *window)
 {
-	GtkNotebook *nb;
-	gboolean handled;
+	GtkWidget *nb;
 
-	nb = GTK_NOTEBOOK (ephy_window_get_notebook (window));
+	nb = ephy_window_get_notebook (window);
 	g_return_if_fail (nb != NULL);
 
-	g_signal_emit_by_name (nb, "change-current-page", -1, &handled);
+	ephy_notebook_prev_page (EPHY_NOTEBOOK (nb));
 }
 
 void
