@@ -27,6 +27,7 @@
 #include "ephy-embed-prefs.h"
 #include "ephy-embed-private.h"
 #include "ephy-file-helpers.h"
+#include "ephy-private.h"
 #include "ephy-shell.h"
 #include "ephy-web-view.h"
 
@@ -250,7 +251,7 @@ main (int argc, char *argv[])
     return -1;
   }
 
-  _ephy_shell_create_instance (FALSE);
+  _ephy_shell_create_instance (EPHY_EMBED_SHELL_MODE_PRIVATE);
 
   server = soup_server_new (SOUP_SERVER_PORT, SERVER_PORT, NULL);
   soup_server_add_handler (server, NULL, server_callback, NULL, NULL);
